<?php

namespace Drupal\custom_entity_pager\Services;

use Drupal\Core\Database\Connection;
use Drupal\Core\Language\LanguageManagerInterface;

/**
 * Contains the logic of the module.
 */
class CustomEntityPager {

  /**
   * The database connection.
   *
   * @var \Drupal\Core\Database\Connection
   */
  private $connection;

  /**
   * The language manager.
   *
   * @var \Drupal\Core\Language\LanguageManagerInterface
   */
  private $languageManager;

  /**
   * Constructor.
   */
  public function __construct(Connection $connection, LanguageManagerInterface $languageManager) {
    $this->connection = $connection;
    $this->languageManager = $languageManager;
  }

  /**
   * Inserts the entity pager.
   *
   * @param string $content_type
   *   Name of the content type to build the pager.
   * @param int $current_nid
   *   Current nid.
   * @param string $field_order
   *   The field to sort by.
   *
   * @return array|null[]
   *   Returns the next and previous items.
   */
  public function getPaginator($content_type, $current_nid, $field_order = '') {
    $result_assoc = $this->getElements($content_type, $field_order);
    return $this->getNextAndPrev($result_assoc, $current_nid);
  }

  /**
   * Gets elements from the database.
   *
   * @param string $content_type
   *   Name of the content type to build the pager.
   * @param string $field_order
   *   The field to sort by.
   *
   * @return array|mixed
   *   Associative array with the entity objects with the id as a key.
   */
  public function getElements($content_type, $field_order) {

    $table_name = '';
    $value_field = '';

    if ($field_order != '') {
      $table_name = 'node__' . $field_order;
      $value_field = $field_order . '_value';
    }

    $id_lang = $this->languageManager->getCurrentLanguage()->getId();

    $connection = $this->connection;
    $query = $connection->select('node_field_data', 'nfd');
    $query->fields('nfd', ['nid', 'title', 'status']);

    if ($table_name != '') {
      $query->join($table_name, $table_name, $table_name . '.entity_id = nfd.nid');
    }

    $query->condition('nfd.type', $content_type);
    $query->condition('nfd.langcode', $id_lang);
    $query->condition('nfd.status', TRUE);

    if ($value_field != '') {
      $query->orderBy($table_name . '.' . $value_field, 'ASC');
    }
    else {
      $query->orderBy('nfd.nid', 'ASC');
    }

    $result = $query->execute();

    return $result->fetchAllAssoc('nid');
  }

  /**
   * Returns the next and previous item.
   *
   * @param array $result_assoc
   *   Associative array with the entity objects with the id as a key.
   * @param int $current_nid
   *   Current nid.
   *
   * @return array|null[]
   *   Returns the next and previous items.
   */
  public function getNextAndPrev($result_assoc, $current_nid) {
    $keys = array_keys($result_assoc);
    $actual_pos = array_search($current_nid, $keys);
    $element_prev = NULL;
    $element_next = NULL;

    if ($actual_pos !== FALSE) {
      // Current node is present in the pager entity list.
      $prev_pos = $actual_pos - 1;
      $next_pos = $actual_pos + 1;

      $limit_array = count($result_assoc) - 1;

      if ($prev_pos >= 0) {
        $element_prev = $result_assoc[$keys[$prev_pos]];
      }

      if ($next_pos <= $limit_array) {
        $element_next = $result_assoc[$keys[$next_pos]];
      }
    }
    else {
      // Current node is missing in the pager entity list.
      $element_next = $result_assoc[$keys[0]];
    }

    return [
      'prev' => $element_prev,
      'next' => $element_next,
    ];
  }

}
